#include "MoodTrainer.h"
#include "BuddyEdit.h"
#include "GetMood.h"
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <stdlib.h>
#include <stdio.h>

MoodTrainer::MoodTrainer (int pollTime, const char * buddyFile, QWidget * parent, const char * name, WFlags fl) : MoodTrainerBase (parent, name, fl) {
  strcpy (tempFile, "moodTrainerTemp.txt");
  strcpy (awFile, "AWOutput");
  diskFullThreshold = 90;
  alarmTime = 600;
  cacheTime = 2147483647;
  strcpy (this->buddyFile, buddyFile);
  this->pollTime = pollTime;

  // Set up buddyList and buddyEditor
  buddyListRefresh();
  buddyEdit = new BuddyEdit (buddyFile, this);
  buddyEdit->hide();

  // Set up timer, but don't start it
  timer = new QTimer(this);

  // Set up moodreader
  getMood = new GetMood(this);
  getMood->hide();
  
  connect(pushButtonStart, SIGNAL(clicked()), this, SLOT(startRunning()));
  connect(pushButtonExit, SIGNAL(clicked()), this, SLOT(done()));
  connect(pushButtonBuddyEdit, SIGNAL(clicked()), this, SLOT(startBuddyEdit()));
  connect(buddyEdit, SIGNAL(buddyListChanged()), this, SLOT(buddyListRefresh()));
  connect(timer, SIGNAL(timeout()), this, SLOT(timerTick()));
  connect(getMood, SIGNAL(choice(int)), this, SLOT(moodChosen(int)));
}

MoodTrainer::~MoodTrainer() {
}

/**
 * Start running simulation!
 */
void MoodTrainer::startRunning() {
  char command[256];

  // Start running the program!

  // Disable the button first
  pushButtonStart->setEnabled(false);

  // Get the timer running!
  timer->start(pollTime * 1000);

  // Run EnvScan and AW in the background
  envMapSize = howLong = awFileNum = 0;

  sprintf(command, "/mmc/mmca1/borglab/groupmedia/EnvScan %d & echo $! > EnvScan.pid", pollTime - 1);

#ifdef DEBUG
  printf ("Executing command: %s\n", command);
#endif

#ifndef X86
  system(command);
#endif

  startAW(awFileNum);
  
}

/**
 * The mood has been chosen!
 */
void MoodTrainer::moodChosen(int mood) {
  char fileName[256];

  // Hide the mood dialog

  getMood->hide();

  // Write data to file

#ifdef DEBUG
  printf ("Mood obtained: %d\n",mood);
#endif

#ifndef X86
  sprintf (fileName, "/mmc//mmca1/data/%s%d.txt", awFile, awFileNum);
  FILE *f = fopen (fileName, "a");
  fprintf (f, "\nMood is: %d\n",mood);
  fclose(f);
#endif

  // Add 1 to awFileNum, startAW again, start timer

  startAW(++awFileNum);
  timer->start(pollTime * 1000);
}

/**
 * Starts AW
 */
void MoodTrainer::startAW (int awFileNum) {
  char command[256];

  sprintf(command, "/mmc/mmca1/borglab/groupmedia/AW /mmc/mmca1/data/%s%d.txt & echo $! > AW.pid", awFile, awFileNum);

#ifdef DEBUG
  printf ("Executing command: %s\n", command);
#endif

#ifndef X86
  system(command);
#endif
}

/**
 * Kills AW
 */
void MoodTrainer::killAW() {
  char command[256];

  sprintf (command, "kill `cat AW.pid 2> /dev/null` 2> /dev/null");

#ifdef DEBUG
  printf ("Executing command: %s\n", command);
#endif

#ifndef X86
  system(command);
#endif
}

/**
 * Starts the buddy editor
 */
void MoodTrainer::startBuddyEdit() {

  buddyEdit->show();
}

/**
 * Close all relevant info to the program, but doesn't close the window yet
 */
void MoodTrainer::closeProg() {
  char command[256];

  // Stop running programs
  timer->stop();

  // Kill EnvScan and AW
  
  sprintf (command, "kill `cat EnvScan.pid 2> /dev/null` 2> /dev/null");

#ifdef DEBUG
  printf ("Executing command: %s\n", command);
#endif

#ifndef X86
  system(command);
#endif

  killAW();

  // Kill pid files

  sprintf (command, "rm -rf EnvScan.pid");

#ifdef DEBUG
  printf ("Executing command: %s\n",command);
#endif
  system(command);

  sprintf (command, "rm -rf AW.pid");

#ifdef DEBUG
  printf ("Executing command: %s\n",command);
#endif

  // Free memory
  for (vector<buddyItem *>::iterator it = buddyList.begin(); it < buddyList.end(); it++) 
    free (*it);
  for (vector<buddyItem *>::iterator it = onlineList.begin(); it < onlineList.end(); it++)
    free (*it);
  for (vector<onlineChecker *>::iterator it = seenID.begin(); it < seenID.end(); it++)
    free(*it);
   
  delete buddyEdit;
  delete timer;
  delete getMood;
}

/**
 * Close all relevant info and close the window
 */
void MoodTrainer::done() {
  closeProg();
  close();
}

/**
 * Gets file size of any file
 */
int MoodTrainer::getFileSize (const char * fileName) {
  // Uses Unix's ls
  
  char command[256]; 
  char temp[256];
  int ans;

  strcpy (command, "la -la ");
  strcat (command, fileName);
  strcat (command, " > ");
  strcat (command, tempFile);

#ifdef DEBUG
  printf ("Executing command: %s\n",command);
#endif
  system (command);

  // Re-open file
  FILE *f = fopen (tempFile, "r");

  if (f == NULL)
    return 0;

  if (fscanf (f, " %s %s %s %s %d", temp, temp, temp, temp, &ans) != 5) 
    ans = 0;

  fclose(f);

  // Delete temp file
  strcpy (command, "rm -f ");
  strcat (command, tempFile);

#ifdef DEBUG
  printf ("Executing command: %s\n",command);
#endif
  system(command);

  return ans;
}

/**
 * Refreshes the buddy list, after it has been modified
 */
void MoodTrainer::buddyListRefresh() {
  char name[256];
  
  // Kill old buddylist
  for (vector<buddyItem *>::iterator it = buddyList.begin(); it < buddyList.end(); it++) {
    buddyItem * nextItem = *it;
    free(nextItem);
  }

  buddyList.clear();

  // Repopulate new buddyList
  FILE * file = fopen (buddyFile, "r");

  while (fgets(name, 9999, file) != NULL) {
    buddyItem *next = (buddyItem *) malloc (sizeof(buddyItem));
    strcpy(next->name, name);
    fgets(next->btID, 9999, file);
    fgets(next->phoneNum, 9999, file);

    buddyList.push_back(next);
  }

  onlineListRefresh();
}

/*
 * Returns 1 if disk is full, 0 otherwise
 */
int MoodTrainer::diskFull(const char * fileSystem) {
  // Use unix's df command
  
  char command[256], temp[256];
  int ans;

  strcpy (command, "df ");
  strcat (command, fileSystem);
  strcat (command, " > ");
  strcat (command, tempFile);

#ifdef DEBUG
  printf ("Executing command: %s\n", command);
#endif
  system(command);

  // Reopen file

  FILE *f = fopen (tempFile, "r");

  fgets (temp, 9999, f);
  if (fscanf (f, " %s %s %s %s %d", temp, temp, temp, temp, &ans) != 5) {
    // Disk if full if we can't even have enough space to write this file!
    ans = 100;
  }

  fclose (f);

  // Delete temp file
  strcpy (command, "rm -f ");
  strcat (command, tempFile);

#ifdef DEBUG
  printf ("Executing command: %s\n", command);
#endif
  system(command);

  if (ans > diskFullThreshold) return 1;
  return 0;
}

/**
 * Refreshes the online List, using the list of seenID(s)
 */
void MoodTrainer::onlineListRefresh() {
  // Clear onlineList
  
  for (vector<buddyItem *>::iterator it = onlineList.begin(); it < onlineList.end(); it++) {
    buddyItem *next = *it;
    free(next);
  }

  onlineList.clear();
  
  for (vector<buddyItem *>::iterator it = buddyList.begin(); it < buddyList.end(); it++) {
    // For each element, we see if it is in the seen ID list 

    buddyItem *next = *it;
    for (vector<onlineChecker *>::iterator it2 = seenID.begin(); it2 < seenID.end(); it2++) {
      onlineChecker *next2 = *it2;

      if (!strcmp (next2->btID, next->btID)) {

	// Add a new buddyItem to onlineList
	// We create a new copy, no aliasing!
	
        buddyItem * newItem = (buddyItem *) malloc (sizeof(buddyItem));
	strcpy(newItem->btID, next->btID);
	strcpy(newItem->name, next->name);
	strcpy(newItem->phoneNum, next->phoneNum);

	// Add item to onlineList, break;
	
	onlineList.push_back(newItem);
	break;
      }
    }
  }

  // Refresh the list box

  listBoxMyBuddies->clear();

  for (vector<buddyItem *>::iterator it = onlineList.begin(); it < onlineList.end(); it++) {
    buddyItem * next = *it;
    listBoxMyBuddies->insertItem(next->name);
  }
}

/**
 * What happens when the timer ticks!
 */
void MoodTrainer::timerTick() {
  char temp[256];
  char btID[256];

  // Check for errors
  
  // Disk full

  int isFull;

#ifdef X86
  isFull = diskFull("/dev/hda1");
#else
  isFull = diskFull("/dev/mmca1");
#endif

  if (isFull) {
    closeProg();
    QMessageBox::critical (this, "Disk is full!", "Fatal error: Disk full.\nTerminating Program.", QMessageBox::Ok, QMessageBox::NoButton);
    close();
  }

  // Same file size for envmap.txt

#ifndef X86
  int fileSize = getFileSize("/mmc/mmca1/data/envmap.txt");
  if (fileSize == envMapSize) {
    howLong += pollTime;
    if (howLong >= alarmTime) {
      closeProg();
      QMessageBox::critical (this, "envmap.txt has not changed!", "Fatal Error:  Not receiving Bluetooth signals!\nTerminating Program.", QMessageBox::Ok, QMessageBox::NoButton);
      close();
    }
  } else {
    envMapSize = fileSize;
    howLong = 0;
  }
#endif

  // Update the list of seenIDs
  
  // Go through every current seenID, update the time taken so far
  vector<onlineChecker *>::iterator startPoint;

  for (startPoint = seenID.begin(); startPoint < seenID.end(); startPoint++) {
    onlineChecker *next = *startPoint;
    next->duration += pollTime;
  }

  // Now check if we have exceeded the cache time
  for (startPoint = seenID.begin(); startPoint < seenID.end(); startPoint++) {
    onlineChecker *next = *startPoint;
    if (next->duration <= cacheTime) break; 
  }

  seenID.erase(seenID.begin(), startPoint);

  // Now add the new seen btIDs
#ifndef X86

  FILE *file = NULL;

  while (file == NULL) {
    file = fopen ("/mmc/mmca1/data/envtmp.txt", "r");
  }

  fgets(temp, 9999, file);
  fgets(temp, 9999, file);
  
  while (fgets(temp, 9999, file) != NULL) {
    if (sscanf (temp, " %s", btID) == 0) break;
    onlineChecker *next = (onlineChecker *) malloc (sizeof(onlineChecker));
    strcpy (next->btID, btID);
    next->duration = 0;
    seenID.push_back(next);
  }

  // Refresh online list
  onlineListRefresh();

#endif

  // Get Mood!
  // Stop timer, kill AW first 

  timer->stop();
  killAW();

  // Run the GetMood dialog box
  getMood->show();
}
