Buffers Files Tools Edit Search Mule C++ Help
/*****************************************************************************
 * File: MoodTrainer.cpp
 * Author: Chris Moh and Anmol Madan, MIT Human Dynamics Group
 * http://groupmedia.media.mit.edu/
 * Copyright (C) 2006 MIT Media Lab
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/





#include <qapplication.h>
#include <qpushbutton.h>
#include <qtextstream.h>
#include <qlistbox.h>
#include <qtimer.h>
#include <qpixmap.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <ZMessageBox.h>
#include "MoodTrainer.h"


//WFlags fl ignored
MoodTrainer::MoodTrainer (int pollTime, int envScanPollTime, const char * buddyFile, QWidget * parent, const char * name, WFlags fl) : ZMainWidget( TRUE, parent, name )
{
  this->runState = FALSE;

  pContent = (ZScrollView *) this->getContentWidget();
  pContent->enableClipper(true);

//  pCST = new QWidget(); 
  pCST = this->getCSTWidget();
  pLeftBtn = new ZPushButton( "CST_Menu", pCST, ZPushButton::JoinRight | ZPushButton::CSTStyle );
  pMidBtn = new ZPushButton( pCST, ZPushButton::JoinLeft | ZPushButton::JoinRight | ZPushButton::CSTStyle );
  pRightBtn = new ZPushButton( "CST_Exit", pCST, ZPushButton::JoinLeft | ZPushButton::CSTStyle );
  
  pCSM = new QPopupMenu( this );
  
  // Add png files for media lab and motorola logos using QPixmap

  idAddBuddy = pCSM->insertItem( "Add Buddy");
  idRemoveBuddy = pCSM->insertItem( "Remove Buddy");
  idOptions = pCSM->insertItem( "Options");
  
  // Use to connect to appropriate slots
  //bool QMenuData::connectItem ( int id, const QObject * receiver, const char * member )

  pLeftBtn->setGeometry( ZDiffR(CstR, Cst3_1R) );
  pLeftBtn->setPopup( pCSM );
  pMidBtn->setGeometry( ZDiffR(CstR, Cst3_2R) );
  pMidBtn->setText( "Start" );
  pRightBtn->setGeometry( ZDiffR(CstR, Cst3_3R) );

  connect( pMidBtn, SIGNAL( clicked() ), this, SLOT( startOrStop() ) );
// TODO: See if we should remove this!
//  connect( pRightBtn, SIGNAL( clicked() ), this, SLOT( done()) );

  // Anmol -- menu slot
  connect( pCSM, SIGNAL( activated(int) ), this, SLOT(slotSelectMenu(int) ) );
  strcpy (tempFile, "moodTrainerTemp.txt");
  diskFullThreshold = 90;
  alarmTime = 1200;
  cacheTime = 600;  // Cache for 10 minutes!
  strcpy (notesFile, "/mmc/mmca1/data/notes.txt");
  timeBetweenAsk = 600;
  lastMoodAskedTime = 0;
  asRunning = FALSE;  // AS is not running!
  strcpy (this->buddyFile, buddyFile);
  this->pollTime = pollTime;
  this->envScanPollTime = envScanPollTime;
  buddySize = onlineSize = seenSize = 0;
  displayLock = 0;

  // Setup Display

  logo = new QPixmap("logom.png");
  logo->resize(30,30);
  logoLabel = new QLabel( pContent->viewport() );
  logoLabel->setPixmap(*logo);
  pContent->addChild(logoLabel,185,5);

  banner = new QPixmap("banner.png");
  banner->resize(100,30);
  bannerLabel = new QLabel( pContent->viewport() );
  bannerLabel->setPixmap(*banner);
  pContent->addChild (bannerLabel, 10, 5); 

  moodLabel = new QLabel( pContent->viewport() );
//  moodLabel = new QLabel(this);

  moodLabel->setText( "Mood: " );
  moodLabel->resize( 150, 24 );
  pContent->addChild(moodLabel, 10, 50);
  
  locLabel = new QLabel( pContent->viewport() );
  locLabel->setText( "Location: " );
  locLabel->resize( 210, 24 );
  pContent->addChild(locLabel, 10, 85);

  buddyLabel = new QLabel( pContent->viewport());
  buddyLabel->setText( "Buddies: " );
  buddyLabel->resize( 80, 24 );
  pContent->addChild(buddyLabel, 10, 120);

  listBoxMyBuddies = new QListBox ( pContent->viewport());
//  listBoxMyBuddies = new QListBox (this);
  listBoxMyBuddies->resize(210, 50);
  pContent->addChild(listBoxMyBuddies, 10, 145); 

  // Set up line editing
  content = new QLineEdit ("Enter notes here!", this);
  content->resize(180, 24);
  pContent->addChild(content, 10, 205);

  writeNote = new QPushButton ("Ok",this);
  writeNote->resize(30,24);
  pContent->addChild(writeNote,200,205);
 
  // Set up timer, but don't start it
  timer = new QTimer(this);

  // New Mood reader
  getMood = new GetMood(this);
  getMood->hide();
  
  connect(timer, SIGNAL(timeout()), this, SLOT(timerTick()));
  connect(getMood, SIGNAL(choice(int)), this, SLOT(moodChosen(int)));

  connect(writeNote, SIGNAL(clicked()), this, SLOT(writeNotes()));

  // Anmol -- Buddy Editing is disabled
  // connect(pushButtonBuddyEdit, SIGNAL(clicked()), this, SLOT(startBuddyEdit()));
  // connect(buddyEdit, SIGNAL(buddyListChanged()), this, SLOT(buddyListRefresh()));
}

MoodTrainer::~MoodTrainer() {
  delete timer;
  delete getMood;
}



//Play audio buzzer
void MoodTrainer::playBuzz(char * pcmfile)
{
  char msg[400];

  if (asRunning==TRUE)
    stopAS();
  sprintf(msg,"/mmc/mmca1/groupmedia/Buzz %s",pcmfile);
  system(msg);

  if (asRunning==TRUE)
    {
      sleep(7);
      startAS();
    }
}
  
/**
 * Starts the audio streamer with a specific filename
 */
void MoodTrainer::startAS() {
  char filename[256],f2[256];
  FILE *f;
  char command[256];

  // start AudioStreamer -- only when buddy spotted

  if (asRunning == TRUE)
    return;
  asRunning = TRUE;

  // Name is defined by the timestamp!
  time_t t = time(NULL); 
  sprintf (filename, "%ld.txt", t);
  f = fopen (notesFile, "a+");
  fprintf(f, "%ld AudioFile %s Opened %s ",t,filename,ctime(&t));
  fclose(f);

  sprintf(command, "/mmc/mmca1/groupmedia/AudioStreamer /mmc/mmca1/data/%s & echo $! > /var/AS.pid",filename);
#ifdef DEBUG
  printf ("Executing : %s\n", command);
#endif
  fflush(stdout);
  system(command);
}

/**
 * Stops the Audio Streamer
 */
void MoodTrainer::stopAS() {
  char command[256];
  // Kill AS as well as the pid file
  if (asRunning == FALSE)
    return;
  asRunning = FALSE;
  sprintf (command, "kill `cat /var/AS.pid 2> /dev/null` 2> /dev/null");
#ifdef DEBUG
  printf ("Executing: %s\n",command);
#endif 
  system(command);
  sprintf (command, "rm /var/AS.pid");
#ifdef DEBUG
  printf ("Executing: %s\n",command);
#endif
  fflush(stdout);
  system(command);

  // Sleep for one second
  sprintf (command, "sleep 1");
#ifdef DEBUG
  printf ("Executing: %s\n",command);
  fflush(stdout);
#endif
  system(command);
}


void MoodTrainer::startOrStop() 
{
  char command[256];

  if (this->runState==FALSE) //Start
    {
      
      //start timer
      timer->start(pollTime * 1000);

      //start EnvScan 
      sprintf(command, "export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/diska/lib ; /mmc/mmca1/groupmedia/EnvScan %d & echo $! > /var/EnvScan.pid", envScanPollTime);
     
#ifdef DEBUG
      printf ("Executing : %s\n", command);
#endif
      system(command);

      pMidBtn->setText( "Stop" );
      runState=TRUE;
      //Load buddylist into BuddyList
      buddyListRefresh();
    }

  else //Stop 
    {
      closeProg();
      pMidBtn->setText( "Start" );
      runState=FALSE;
    }

}

void MoodTrainer::closeProg() {

  char command[256];

  // Stop timer
  timer->stop();

  // Kill EnvScan
  sprintf (command, "kill `cat /var/EnvScan.pid 2> /dev/null` 2> /dev/null");
#ifdef DEBUG
  printf ("Executing command: %s\n",command);
  fflush(stdout);
#endif
  system(command);

  // RM pid files for EnvScan
  sprintf (command, "rm /var/EnvScan.pid");
#ifdef DEBUG
  printf ("Executing command: %s\n",command);
  fflush(stdout);
#endif
  system(command);

  // Remove AS
  stopAS();

  // Anmol --check

  // Free memory
  for (int i = 0; i < buddySize; i++)
    free(buddyList[i]);
  for (int i = 0; i < onlineSize; i++)
    free(onlineList[i]);
  for (int i = 0; i < seenSize; i++)
    free(seenID[i]);
  buddySize = onlineSize = seenSize = 0;

  //Anmol--check   
  // delete buddyEdit;
  // delete timer;
  // delete getMood;
}

/**
 * What happens when the timer ticks!
 */
void MoodTrainer::timerTick() {
  char temp[256];
  char btID[256];

  // Check for errors
  
  // Disk full
  int isFull;
#ifdef X86
  isFull = diskFull("/dev/hda1");
#else
  isFull = diskFull("/dev/mmca1");
#endif

  if (isFull) {
    closeProg();
    ZMessageBox::information (this, 0, "Fatal Error: \n TransFlash is Full","OK",NULL,NULL);
    close();
  }

// Same file size for envmap.txt
#ifndef X86
  int fileSize = getFileSize("/mmc/mmca1/data/envmap.txt");
  if (fileSize == envMapSize) {
    howLong += pollTime;
    if (howLong >= alarmTime) {
      closeProg();
      ZMessageBox::information(this, 0, "Fatal Error:  \n envmap.txt unchanged,check blueZ / TAPI ", "OK", NULL, NULL);
      close();
    }
  } else {
    envMapSize = fileSize;
    howLong = 0;
  }
#endif

  // Update the list of seenIDs
  // Go through every current seenID, update the time taken so far
  int i;

  for (i = 0; i < seenSize; i++) {
    onlineChecker *next = seenID[i];
    next->duration += pollTime;
  }

  // Now check if we have exceeded the cache time
  for (i = 0; i < seenSize; i++) {
    onlineChecker *next = seenID[i];
    if (next->duration <= cacheTime) break; 
  }

  // Erase if needed
  // Anmol -- check
  for (int j = 0; j < seenSize - i; j++) {
    if (j - i < 0)
      free(seenID[j]);
    seenID[j] = seenID[j+i];
  }

  seenSize -= i;

  // We create a list of people previously on the onlineList
  int numPeople = onlineSize;
  char people[1000][32];

  for (int i = 0; i < onlineSize; i++)
    strcpy(people[i], onlineList[i]->btID);

  // Now add the new seen btIDs
#ifndef X86

  FILE *file = NULL;

  while (file == NULL) {
    file = fopen ("/mmc/mmca1/data/envtmp.txt", "r");
  }

  fgets(temp, 9999, file);
  fgets(temp, 9999, file);

  // Get location info
  int loc[5];
  char buf[1024];

  sscanf (temp, " %d %d %d %d %d", loc,loc+1,loc+2,loc+3,loc+4);
  sprintf (buf, "Location: %d %d %d %d %d", loc[0],loc[1],loc[2],loc[3],loc[4]);

  locLabel->setText(buf);
  
  while (fgets(temp, 9999, file) != NULL) {
    if (sscanf (temp, " %s", btID) == 0) break;
    printf ("Seen ID: %s\n",btID);
    onlineChecker *next = (onlineChecker *) malloc (sizeof(onlineChecker));
    strcpy (next->btID, btID);
    next->duration = 0;
    seenID[seenSize] = next;
    seenSize++;
  }

  // Refresh online list
  onlineListRefresh();

#endif

  
  if (asRunning == FALSE && onlineSize != 0) {
    // If it is not running, and there are people online, we start the program!

    if (numPeople == 0) {
      // If previous number of people was 0, ask them to wear the headset
      if (!displayLock) {
	displayLock = 1;
	playBuzz("plsheadset.pcm");
        ZMessageBox::information(this, 0, "Please wear your headset", "OK", NULL, NULL);
	displayLock = 0;
      }
    }
    startAS();
  } else if (asRunning == TRUE && onlineSize == 0) {
    // Stop AS if no one is online!

    if (numPeople != 0) {
      // If previous number of people wasn't 0, inform them that no more friends are available
      if (!displayLock) {
	displayLock = 1;
        ZMessageBox::information(this, 0, "You may remove your headset.", "OK", NULL, NULL);
	displayLock = 0;
      }
    }
    stopAS();
  }

  // Run the GetMood dialog box
  // Check for commonality in previous and current online lists
  int run = FALSE;

  for (int i = 0; i < numPeople; i++)
    for (int j = 0; j < onlineSize; j++) {
      if (!strcmp(people[i], onlineList[j]->btID)) {
	printf ("Buddy: %s\n",onlineList[j]->name);
	run = TRUE;
	break;
      }
    }
  if (run == TRUE && !displayLock) {
    // Check when the last mood was chosen?
    if (time(NULL) - lastMoodAskedTime < timeBetweenAsk)
      return;   
    printf("Time Comparison %ld %ld \n", time(NULL) - lastMoodAskedTime, timeBetweenAsk);
    displayLock = 1;
    playBuzz("plsanswer.pcm");
    getMood->show();

  }
}


/**
 * Default: writes to notes.txt!
 * The mood has been chosen!
 */
void MoodTrainer::moodChosen(int mood) {
  char buf[256];
  char moods[5][20] = {"Excited", "Interested", "Neutral", "Bored", "Depressed"};

  lastMoodAskedTime = time(NULL); 

  // Hide the mood dialog

  getMood->hide();
  displayLock = 0;

  // Kill AS
  // stopAS();

  // Write data to file

#ifdef DEBUG
  printf ("Mood obtained: %s\n",moods[mood]);
#endif

#ifndef X86
  FILE *f = fopen (notesFile, "a+");
  printf ("File Opened to write mood: %s\n",notesFile);
  if (f == NULL) {
    printf ("File could not be opened!");
  }
  fflush(stdout);
  fprintf (f, "%ld ",lastMoodAskedTime);
  fprintf (f, " Mood  %s",moods[mood]);
  fprintf (f, " Buddies ");
  for (int i = 0; i < onlineSize; i++)
    fprintf (f, " %s ",onlineList[i]->name);
  fprintf(f," %s ",ctime(&lastMoodAskedTime));
  fclose(f);
#endif
  // Change label
  sprintf (buf, "Mood: %s",moods[mood]);
  moodLabel->setText(buf);

  // Add 1 to asFileNum, startAS again, start timer

  timer->start(pollTime * 1000);
  //startAS();
}


/**
 * Starts the buddy editor
 */
/*void MoodTrainer::startBuddyEdit() {

  buddyEdit->show();
  }*/

/**
 * Close all relevant info and close the window
 */
void MoodTrainer::done() {
  closeProg();
  close();
}

/**
 * Gets file size of any file
 */
int MoodTrainer::getFileSize (const char * fileName) {
  // Uses Unix's ls
  
  char command[256]; 
  char temp[256];
  int ans;

  strcpy (command, "ls -la ");
  strcat (command, fileName);
  strcat (command, " > ");
  strcat (command, tempFile);

#ifdef DEBUG
  printf ("Executing command: %s\n",command);
#endif
  system (command);

  // Re-open file
  FILE *f = fopen (tempFile, "r");

  if (f == NULL)
    return 0;

  if (fscanf (f, " %s %s %s %s %d", temp, temp, temp, temp, &ans) != 5) 
    ans = 0;

  fclose(f);

  // Delete temp file
  strcpy (command, "rm -f ");
  strcat (command, tempFile);

#ifdef DEBUG
  printf ("Executing command: %s\n",command);
#endif
  system(command);
  return ans;
}

/**
 * Refreshes the buddy list, after it has been modified
 */

void MoodTrainer::buddyListRefresh() {
  char name[256];
  
  // Kill old buddylist
  //  for (int i = 0; i < buddySize; i++)
  //  free(buddyList[i]);

  buddySize = 0;

  // Repopulate new buddyList
  FILE * file = fopen (buddyFile, "r");

  if (file == NULL) {
    printf ("No buddy file exists.\n");
    printf ("Will not update buddy information.\n");
    return;
  }
  while (fgets(name, 9999, file) != NULL) {
    buddyItem *next = (buddyItem *) malloc (sizeof(buddyItem));
    strcpy(next->name, name);
    fgets(next->btID, 9999, file);
    next->btID[strlen(next->btID)-1] = 0;
    fgets(next->phoneNum, 9999, file);

    buddyList[buddySize] = next;
    buddySize++;
  }
  fclose(file);

  // check
  onlineListRefresh();
}


/*
 * Returns 1 if disk is full, 0 otherwise
 */
int MoodTrainer::diskFull(const char * fileSystem) {
  // Use unix's df command
  
  char command[256], temp[256];
  int ans;

  strcpy (command, "df ");
  strcat (command, fileSystem);
  strcat (command, " > ");
  strcat (command, tempFile);

#ifdef DEBUG
  printf ("Executing command: %s\n", command);
#endif
  system(command);

  // Reopen file

  FILE *f = fopen (tempFile, "r");

  fgets (temp, 9999, f);
  if (fscanf (f, " %s %s %s %s %d", temp, temp, temp, temp, &ans) != 5) {
    // Disk if full if we can't even have enough space to write this file!
    ans = 100;
  }

  fclose (f);

  // Delete temp file
  strcpy (command, "rm -f ");
  strcat (command, tempFile);

#ifdef DEBUG
  printf ("Executing command: %s\n", command);
#endif
  system(command);

  if (ans > diskFullThreshold) return 1;
  return 0;
}

/**
 * Refreshes the online List, using the list of seenID(s)
 */
void MoodTrainer::onlineListRefresh() {
  // Clear onlineList
  
  for (int i = 0; i < onlineSize; i++)
    free(onlineList[i]);

  onlineSize = 0;
  
  for (int i = 0; i < buddySize; i++) {
    // For each element, we see if it is in the seen ID list 

    buddyItem *next = buddyList[i];
    printf ("Checking: %s with btID: %s(check)\n",next->name, next->btID);
    fflush(stdout);
    for (int j = 0; j < seenSize; j++) {
      onlineChecker *next2 = seenID[j];
      
      if (!strcmp (next2->btID, next->btID)) {
	printf ("Buddy seen: %s with btID: %s\n",next->name, next->btID);
	
	fflush(stdout);
	
	// Add a new buddyItem to onlineList
	// We create a new copy, no aliasing!
	
        buddyItem * newItem = (buddyItem *) malloc (sizeof(buddyItem));
	strcpy(newItem->btID, next->btID);
	strcpy(newItem->name, next->name);
	strcpy(newItem->phoneNum, next->phoneNum);
	
	// Add item to onlineList, break;
	onlineList[onlineSize] = newItem;
	onlineSize++;
	break;
      }
      
    }
  }

  // Refresh the list box
  listBoxMyBuddies->clear();

  if (onlineSize>0) {
    
    char buf[1000];
    time_t t = time(NULL);
    sprintf(buf, "%ld Buddies ",t);
    
    for (int mm=0;mm<onlineSize;mm++)
      strcat(buf, onlineList[mm]->name);
    
    strcat(buf," ");
    strcat(buf,ctime(&t));
    
    FILE *f = fopen (notesFile, "a+");
    fprintf (f, "%s", buf);
    fclose(f);
  }	
  
  for (int i = 0; i < onlineSize; i++) {
    buddyItem * next = onlineList[i];
    listBoxMyBuddies->insertItem(next->name);
  }
}


//decide if needed
void MoodTrainer::slotSelectMenu( int idMenu)
{
  /*    if( idMenu == idSend )
    {
        if( bJustSend == FALSE)
        {
            bJustSend = TRUE;
            pCSM->setItemChecked( idSend,  bJustSend );
            pCSM->setItemChecked( idSendWithResult,  !bJustSend );
            pMidBtn->setText( "Send the SMS" );
        }
    }
    else if( idMenu == idSendWithResult )
    {
        if( bJustSend == TRUE)
        {
            bJustSend = FALSE;
            pCSM->setItemChecked( idSend,  bJustSend );
            pCSM->setItemChecked( idSendWithResult,  !bJustSend );
            pMidBtn->setText( "Send with result" );
        }
    }

  */

}

/**
 * Writes notes to file
 * default:  writes to notes.txt!
 */
void MoodTrainer::writeNotes() {
  char buf[1000];

  time_t t = time(NULL);
  sprintf (buf, "%ld Notes %s %s",t, content->text().latin1(),ctime(&t));
  printf ("%s",buf);
  
  // Clear the notes
  content->setText("");

  fflush(stdout);

  FILE *f = fopen (notesFile, "a+");
  fprintf (f, "%s", buf);
  fclose(f);
}
